﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Random;

namespace HeuristicLab.Algorithms.ALPS.SteadyState {
  [Item("RandomLayerProcessor", "An operator which applies a specified operator on a random selected sub-scope (layer).")]
  [StorableClass]
  public class RandomLayerProcessor : SingleSuccessorOperator, IStochasticOperator {
    private OperatorParameter OperatorParameter {
      get { return (OperatorParameter)Parameters["Operator"]; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }

    public IOperator Operator {
      get { return OperatorParameter.Value; }
      set { OperatorParameter.Value = value; }
    }

    [StorableConstructor]
    private RandomLayerProcessor(bool deserializing) : base(deserializing) { }
    private RandomLayerProcessor(RandomLayerProcessor original, Cloner cloner)
      : base(original, cloner) {
    }
    public RandomLayerProcessor()
      : base() {
      Parameters.Add(new OperatorParameter("Operator", "The operator which should be applied on a random sub-scopes of the current scope."));
      Parameters.Add(new LookupParameter<IRandom>("Random", "The random number generator to use."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RandomLayerProcessor(this, cloner);
    }

    public override IOperation Apply() {
      var random = RandomParameter.ActualValue;

      var next = new OperationCollection(base.Apply());
      if (Operator != null) {
        var subScopes = ExecutionContext.Scope.SubScopes;
        int index = random.Next(subScopes.Count);
        next.Insert(0, ExecutionContext.CreateOperation(Operator, subScopes[index]));
      }
      return next;
    }
  }
}